#!/bin/bash
# _/_/_/_/ _/_/_/_/ _/_/_/_/ _/_/_/_/ _/_/_/_/ _/_/_/_/ _/_/_/_/ _/_/_/_/
# Name          : heartbeat_logmoni.sh
# Description   : heartbeat ロギングデーモンのプロセス監視を行う
# Author        : 2006.10.19 h.yamauchi
# Editor        : 2007.02.09 t.fukada
# Editor        : 2007.03.01 m.takeuchi
# Editor        : 2009.08.31 h.yamauchi
# Source        : /etc/ha.d/monitoring/heartbeat_logmoni.sh
# OS  Version   : RHEL ES4 Update4, RHEL 5.3
# HB2 Version   : 2.0.8.p1, 2.1.3.p1, 2.1.4-1
# Script Version : 2.00
#
# License: GNU General Public License (GPL)
# Copyright (c) 2008 NIPPON TELEGRAPH AND TELEPHONE CORPORATION
# _/_/_/_/ _/_/_/_/ _/_/_/_/ _/_/_/_/ _/_/_/_/ _/_/_/_/ _/_/_/_/ _/_/_/_/

LOG_PID_FILE=/var/run/logd.pid
MINFO=/proc/meminfo

# 監視するHeartbeat LoggingDaemonプロセス
MONITORING_LOGD=("ha_logd: read process" "ha_logd: write process")

# Heartbeatロギングデーモンモニタ プロセス
KILLING_SELF_PROCESS=(heartbeat_logmoni.sh)

# ロギングデーモン監視開始待ち時間(Default:30)
MON_LOGD_START_WAIT_TIME=30

# ロギングデーモン監視間隔(Default:60)
MON_LOGD_MONITORING_WAIT_TIME=30

# ロギングデーモン監視ログファイル
MON_LOGD_MONITORING_HBLOG=/var/log/heartbeat_process_monitor.log

# ログメッセージ
LPPID="Heartbeat Logging Daemon Monitor($$)"
INF="INFO"
ERR="ERROR"
WAN="WARNING"
ILOG001="${INF}: monitoring [start]"
ILOG002="${INF}: monitoring [beginning waiting]"
ILOG003="${INF}: monitoring [end]"
ILOG004="${INF}: system reboot now"
ILOG005="${INF}: monitoring [process is alive]"
ILOG006="${INF}: logd_restart start"
ILOG007="${INF}: logd_restart start: vmstat --"
ILOG008="${INF}: logd_restart start: pid --"
ILOG009="${INF}: logd_restart start: procfile --"
ILOG010="${INF}: logd_restart start: /var/run/logd.pid, not exist"
ILOG011="${INF}: logd_restart start: minfo --"
ILOG012="${INF}: logd_restart end"
ELOG001="${ERR}: target monitor logd is down -- "
ELOG002="${ERR}: perl is necessary for heartbeat_logmoni.sh"
WLOG001="${WAN}: killing heartbeat logd -- "
WLOG002="${WAN}: monitoring [logd is restart]"

#
# ロギング
#
logging(){
    echo `date` ${LPPID} ${1} >> ${MON_LOGD_MONITORING_HBLOG}

    # for ocf_log
    #ocf_log debug ${LPPID} ${1} >> ${HALOG_FILE} 2>&1
}
#
# EXIT trap and logging
#
on_signal0_recieve(){
    logging "${ILOG003}"
} 
#
# モニタ初期化
#
logd_monitor_start(){

    #
    # 開始時にこのスクリプトが既に実行中の場合は停止する
    #
    for KILL_SELF_PROC in "${KILLING_SELF_PROCESS[@]}"
    do
        ps -ef | grep "${KILL_SELF_PROC}" | grep -v grep | \
          awk -v PID=$$ '$2!=PID {print $2}' | xargs kill -9 > /dev/null 2>&1
    done

    #
    # heartbeatからのシグナルを拒否
    #
    trap '' 1 2 3 
    trap  on_signal0_recieve 0
    #
    logging "${ILOG002} ${MON_LOGD_START_WAIT_TIME}sec"
    sleep $MON_LOGD_START_WAIT_TIME
}

#
# モニタリング対象プロセスの生存確認
#
isalive_logd(){

    # モニタリング対象プロセスの生存を確認する
    ISALIVE_FLG=1

    for MON_LOGD in "${MONITORING_LOGD[@]}"
    do
        pgrep -fl "${MON_LOGD}" > /dev/null 2>&1
        PRESULT=$?
        if [ ${PRESULT} -ne 0 ]; then
            logging "${ELOG001} ${MON_LOGD}"
            ISALIVE_FLG=0
            break;
        fi
    done

    return ${ISALIVE_FLG}
}

#
# 全ＨＢロギングデーモンプロセス関連ＫＩＬＬ
#
logd_killall(){

    # HeartBeat ロギングデーモン系のKILL
    for KILL_PROC in "${MONITORING_LOGD[@]}"
    do
        pgrep -fl "${KILL_PROC}" > /dev/null 2>&1
        RESULT=$?
        if [ ${RESULT} -eq 0 ]; then
            KILL_PID=`pgrep -fl "${KILL_PROC}" | awk '{print $1}'`
            logging "${WLOG001} ${KILL_PROC}(pid:${KILL_PID})"
            echo ${KILL_PID} | xargs kill -9 > /dev/null 2>&1
        fi
    done
}

#
# ロギングデーモンのリスタート
#
logd_restart(){
    logging "${ILOG006}"

    # check vmstat,pidfile for ha_logd
    if  [ "x${VSTAT_BIN}" != "x" ]; then
        logging "${ILOG007}:`${VSTAT_BIN}`"
    fi

    if [ -f ${LOG_PID_FILE} ]; then
        LOGDPID=`cat ${LOG_PID_FILE} | tr -d ' '`
        logging "${ILOG008} ${LOG_PID_FILE}, exist. pid:${LOGDPID}"
        LOGDPROCFILE="/proc/${LOGDPID}/cmdline"
        if [ -f ${LOGDPROCFILE} ]; then
            CMD=`cat ${LOGDPROCFILE}`
            logging "${ILOG009} ${LOGDPROCFILE}, exist. cmd:${CMD}"
        else
            logging "${ILOG009} ${LOGDPROCFILE}, not exist"
        fi
    else
        logging "${ILOG010}"
    fi

    if [ -f ${MINFO} ]; then
        MMINFO=`cat ${MINFO}`
        logging "${ILOG011} ${MMINFO}"
    fi

    #status check for ha_logd
    ${HA_BIN}/ha_logd -s >> ${MON_LOGD_MONITORING_HBLOG} 2>&1
    rtn1=$?

    #restart for ha_logd
    ${PERLS_BIN} <<EOT
setpgrp;
system("${HA_BIN}/ha_logd -d >> ${MON_LOGD_MONITORING_HBLOG} 2>&1");
EOT
    rtn2=$?
    logging "${WLOG002} rtn1:$rtn1 rtn2:$rtn2"

    if  [ "x${VSTAT_BIN}" != "x" ]; then
        logging "${ILOG007}:`${VSTAT_BIN}`"
    fi

    logging "${ILOG012}"
}

#
# モニタリング
#
logd_monitor(){

    logd_monitor_start
    logging "${ILOG001}"

    #
    # Monitoringループ
    #
    while [ 1 ]
    do
        # モニタリング対象プロセスの生存を確認する
        isalive_logd
        ISALIVE_FLG=$?

        if [ $ISALIVE_FLG -eq 0 ]; then

            # モニタリング対象プロセスが存在していない場合

            # HeartBeat ロギングデーモン系のKILL
            logd_killall

            # ロギングデーモンを再起動する
            logd_restart

        else

            # モニタリング対象プロセスが存在している場合
            logging "${ILOG005}"

        fi

        #
        # モニタリング間隔の間、スリープ
        #
        sleep $MON_LOGD_MONITORING_WAIT_TIME
    done

    logging "${ILOG003}"
    exit 0
}

#
# Usage
#
logd_usage(){
    echo "Usage : $KILLING_SELF_PROCESS { monitor | killall } " 1>&2
    return 1
}

#
# validate
#
PERLS_BIN=`which perl`
if [ $? -ne 0 ]; then
    logging "${ELOG002}"
    exit 1
fi
VSTAT_BIN=`which vmstat`
if [ $? -ne 0 ]; then
    VSTAT_BIN=""
fi
#
#
#
case $1 in
    monitor)    logd_monitor
        exit $? ;;
    killall)    logd_killall
        exit $? ;;
    *)          logd_usage
        exit $? ;;
esac
exit 0
